rm(list=ls())

require(nlme)
require(rpart)
require(rpart.plot)
require(lme4)
require(dplyr)
require(lattice)
require(ggplot2)


source("mymixedtree.R")
source("GMERT.R")
source("Speiser_BiMM_Tree.R")

dati <- read.csv("dataset_PoliMi.csv")

# divisione del dataset in training set e test set
set.seed(4)
test <- sample(1:dim(dati)[1],round(dim(dati)[1]*0.2)) # 5584 osservazioni
train_dataset <- dati[-test,] #13.028 osservazioni
head(train_dataset)
names(train_dataset)
test_dataset <- dati[test,] #5584 osservazioni

# prova funzione intera
 GMET <- mymixedtree(formula = Status ~ Sex +Nationality +PrevStudies +
                     AdmissionScore +AccessAge +WeiAvgEval1.1 +AvgAtt11 +
                     TotalCredits1.1, dataset = train_dataset,  random = ~ (1|DegreeName.out),
                   family = binomial, tree.control = rpart.control(cp = 0.006, maxdepth=10), cv = F)
 rpart.plot(GMET$Tree)
 rand_eff = GMET$EffectModel
 GMET$RandomEffects



# plot random effect
rand_intercept = ranef(GMET$EffectModel, condVar=TRUE)
dotplot(rand_intercept,strip=T, lty= 4)
rand_intercept

# VPC 
psiA = as.numeric(summary(GMET$EffectModel)$varcor)
psiA/(psiA +pi^2/3)

predict.GMET <- function(object, newdata, type = "response", ...){
  # get underlying tree structure
  tree.aux = object$Tree
  # substitute the row index as predicted value in each leaf
  tree.aux$frame$yval = as.numeric(rownames(tree.aux$frame))
  # get the leaf indicator for each newdata
  newdata[,"nodeInd"] <- predict(tree.aux, newdata= newdata)
  #predict using GLMM
  predict(object$EffectModel, newdata = newdata, type = type,...)
}

# ROC analysis
test_pred <- predict(GMET,test_dataset,re.form=NULL,type="response")
p0 = seq(0,1,0.001)
sensitivity <- specificity_comp <- NULL
i=0
for(k in p0){
  i = i+1
  pred2 <- ifelse(test_pred > p0[i],"Graduate","Dropout")
  t11 = length(which(pred2 =="Graduate"& test_dataset$Status=="Graduate"))
  t10 = length(which(pred2 =="Graduate" & test_dataset$Status=="Dropout"))
  t01 = length(which(pred2 =="Dropout" & test_dataset$Status=="Graduate"))
  t00 = length(which(pred2 =="Dropout" & test_dataset$Status=="Dropout"))
  specificity_comp[i] <- 1 - t11/(t11+t01)
  sensitivity[i] <- t00/(t00+t10) 
}
plot(specificity_comp, sensitivity, type = "l", xlab='1 - specificity', cex.lab = 1.3)
lines(seq(0,1,0.01), seq(0,1,0.01), lty = "dashed")
i0 = 601 # best ROC value
points(specificity_comp[i0], sensitivity[i0], pch = 1)
p0opt = p0[i0] # threshold value
p0opt

library(PRROC)
PRROC_obj <- roc.curve(scores.class0 = test_pred, weights.class0=ifelse(test_dataset$Status=='Graduate',1,0),
                       curve=TRUE)
plot(PRROC_obj)



# ONE SHOT indexes
test_pred_class <- ifelse(test_pred > p0opt,1,0)
predicted = factor(test_pred_class, levels = c(0,1),labels = c('Dropout','Graduate'))
observed = factor(test_dataset$Status)
misc.table = table(predicted, observed)
misc.table

TP = misc.table[1,1]
TN = misc.table[2,2]
FP = misc.table[2,1]
FN = misc.table[1,2]
tot = dim(test_dataset)[1]

misClasificError <- (FP+FN)/tot
print(paste('Accuracy',round(1-misClasificError,4)))
print(paste('Sensitivity',round(TP/(TP+FN),4)))
print(paste('Specificity',round(TN/(TN+FP),4)))

# Multi shot
sensitivity_j <- specificity_j <- accuracy_j <- NULL
for( j in 1:20){
  test_j <- sample(1:dim(dati)[1],round(dim(dati)[1]*0.1)) # 5584 osservazioni
  train_j <- dati[-test,] #13.028 osservazioni
  test_dataset_j <- dati[test_j,]
  m_j <- mymixedtree(Status ~ Sex +Nationality +PrevStudies +
                       AdmissionScore +AccessAge +WeiAvgEval1.1 +AvgAtt11 +
                       TotalCredits1.1, dataset = train_j,  random = ~ (1|DegreeName.out),
                     family = binomial, cp = 0.005, cv = F)
  test_pred_j <- predict(m_j,test_dataset_j,re.form=NULL,type="response")
  test_pred_class_j <- ifelse(test_pred_j > p0opt,1,0)
  predicted_j = factor(test_pred_class_j, levels = c(0,1),labels = c('Dropout','Graduate'))
  observed_j = factor(test_dataset_j$Status)
  misc.table_j = table(predicted_j, observed_j)
  TP_j = misc.table_j[1,1]
  TN_j = misc.table_j[2,2]
  FP_j = misc.table_j[2,1]
  FN_j = misc.table_j[1,2]
  tot_j = dim(test_dataset_j)[1]
  accuracy_j[j] <- 1 - (FP_j+FN_j)/tot_j
  sensitivity_j[j] <- TP_j/(TP_j+FN_j)
  specificity_j[j] <- TN_j/(TN_j+FP_j)
}
avg_accuracy = mean(accuracy_j)
avg_accuracy 
avg_sensitivity = mean(sensitivity_j)
avg_sensitivity 
avg_specificity = mean(specificity_j)
avg_specificity 
sd(accuracy_j)
sd(sensitivity_j)
sd(specificity_j)



## Comparison with GMERT function
# GMERT
prop.table(table(train_dataset$Status))
dati_gmert = train_dataset
test_gmert = test_dataset

xnam = c("Sex","Nationality","PrevStudies","AdmissionScore","AccessAge","WeiAvgEval1.1",
         "AvgAtt11","TotalCredits1.1")
ni = as.vector(table(train_dataset$DegreeName.out))
tni = as.vector(table(test_dataset$DegreeName.out))
Xi = list()
tXi = list()
Zi = list()
tZi = list()
Yi = list()
tYi = list()

dati_gmert$Status = ifelse(dati_gmert$Status=="Graduate",1,0)
test_gmert$Status = ifelse(test_gmert$Status=="Graduate",1,0)
prop.table(table(dati_gmert$Status))
prop.table(table(test_gmert$Status))

for(i in 1:length(unique(train_dataset$DegreeName.out))){
  Xi[[i]] = dati_gmert[dati_gmert$DegreeName.out==levels(train_dataset$DegreeName.out)[i],c(6,8,9,13,15,33,35,36)]
  tXi[[i]] = test_gmert[test_gmert$DegreeName.out==levels(test_dataset$DegreeName.out)[i],c(6,8,9,13,15,33,35,36)]
    Zi[[i]]= as.matrix(rep(1,ni[i]))
    tZi[[i]]= as.matrix(rep(1,tni[i]))
    Yi[[i]] = as.matrix(dati_gmert$Status[dati_gmert$DegreeName.out==levels(train_dataset$DegreeName.out)[i]])
    tYi[[i]] = as.matrix(test_gmert$Status[test_gmert$DegreeName.out==levels(train_dataset$DegreeName.out)[i]])
}

q=1
bizero <- list()
length(bizero)<- 19		
for(i in 1:19)bizero[[i]] <- matrix(0,nrow=q,ncol=1)


gmert_mod = gmert(ni,  Xi=Xi, Zi=Zi,  Yi=Yi, tni=tni, tXi=tXi, tZi=tZi, tYi=tYi, xnam=xnam ,usesurrogate=2
                  ,surrogatestyle=0,maxsurrogate=0,maxcompete=2,minsplit=30,minbucket=10,maxdepth=4,xval=10
                  ,cp=0.006
                  ,F=1			#use F=50 to reproduce results in HAJJEM, et al.  # Chiara: micro-iter mi sembrano F*4
                  ,s.change=1e-6
                  ,mert.niter= 10		#use mert.niter=200 to reproduce results in HAJJEM, et al.
                  ,L=1			#use L=50  to reproduce results in HAJJEM, et al.
                  ,mert.verbose = TRUE ,sigmasqzero=1,Dzero=diag(x = 0.01 ,nrow=q,ncol=q),bizero=bizero	,family=binomial 		
                  ,rc= 1e-4,gmert.niter = 1		
                  ,S=1			#use S=55 to reproduce results in HAJJEM, et al.						
                  ,verbose = TRUE)


rpart.plot(gmert_mod[[1]]$gmert.tree)

gmert_mod[[1]]$gmert.rf
gmert_mod[[1]]$rf.mert
gmert_mod[[1]]$cvcr
gmert_mod[[1]]$gmert.nb.leaves
gmert_mod[[1]]$gmert.b1

rand_intercept_gmert = rand_intercept
rand_intercept_gmert$DegreeName.out$`(Intercept)` = gmert_mod[[1]]$gmert.b1
dotplot(rand_intercept_gmert,strip=T, lty= 4)

plot(as.vector(rand_intercept$DegreeName.out$`(Intercept)`), as.vector(rand_intercept_gmert$DegreeName.out$`(Intercept)`), type='l')
cor.test((rand_intercept$DegreeName.out$`(Intercept)`), (rand_intercept_gmert$DegreeName.out$`(Intercept)`))

mupred = gmert_mod[[1]]$gmert.t.c.prob
ypred = as.integer(mupred>p0opt)


# VPC 
psiA = (gmert_mod[[1]]$cvcr)^2
psiA/(psiA +pi^2/3)



# ROC analysis


test_dataset_ord = test_dataset[order(test_dataset$DegreeName.out),]
test_pred <- mupred
p0 = seq(0,1,0.001)
sensitivity <- specificity_comp <- NULL
i=0
for(k in p0){
  i = i+1
  pred2 <- ifelse(test_pred > p0[i],"Graduate","Dropout")
  t11 = length(which(pred2 =="Graduate"& test_dataset_ord$Status=="Graduate"))
  t10 = length(which(pred2 =="Graduate" & test_dataset_ord$Status=="Dropout"))
  t01 = length(which(pred2 =="Dropout" & test_dataset_ord$Status=="Graduate"))
  t00 = length(which(pred2 =="Dropout" & test_dataset_ord$Status=="Dropout"))
  specificity_comp[i] <- 1 - t11/(t11+t01)
  sensitivity[i] <- t00/(t00+t10) 
}
plot(specificity_comp, sensitivity, type = "l", xlab='1 - specificity', cex.lab = 1.3)
lines(seq(0,1,0.01), seq(0,1,0.01), lty = "dashed")
i0 = 601 # best ROC value
points(specificity_comp[i0], sensitivity[i0], pch = 1)
p0opt = p0[i0] # threshold value
p0opt

library(PRROC)
PRROC_obj <- roc.curve(scores.class0 = test_pred, weights.class0=ifelse(test_dataset_ord$Status=='Graduate',1,0),
                       curve=TRUE)
plot(PRROC_obj)



# ONE SHOT indexes
test_pred_class <- ifelse(test_pred > p0opt,1,0)
predicted = factor(test_pred_class, levels = c(0,1),labels = c('Dropout','Graduate'))
observed = factor(test_dataset_ord$Status)
misc.table = table(predicted, observed)
misc.table

TP = misc.table[1,1]
TN = misc.table[2,2]
FP = misc.table[2,1]
FN = misc.table[1,2]
tot = dim(test_dataset)[1]

misClasificError <- (FP+FN)/tot
print(paste('Accuracy',round(1-misClasificError,4)))
print(paste('Sensitivity',round(TP/(TP+FN),4)))
print(paste('Specificity',round(TN/(TN+FP),4)))

# Multi shot
dati_n = dati[order(dati$DegreeName.out),]
sensitivity_j <- specificity_j <- accuracy_j <- NULL
for( j in 1:20){
  test_j <- sample(1:dim(dati_n)[1],round(dim(dati_n)[1]*0.1)) # 5584 osservazioni
  train_j <- dati_n[-test,] #13.028 osservazioni
  test_dataset_j <- dati_n[test_j,]
  
  
  ni = as.vector(table(train_j$DegreeName.out))
  tni = as.vector(table(test_dataset_j$DegreeName.out))
  Xi = list()
  tXi = list()
  Zi = list()
  tZi = list()
  Yi = list()
  tYi = list()
  
  dati_gmert = train_j
  test_gmert = test_dataset_j
  dati_gmert$Status = ifelse(dati_gmert$Status=="Graduate",1,0)
  test_gmert$Status = ifelse(test_gmert$Status=="Graduate",1,0)
  
  for(i in 1:length(unique(train_dataset$DegreeName.out))){
    Xi[[i]] = dati_gmert[dati_gmert$DegreeName.out==levels(train_dataset$DegreeName.out)[i],c(6,8,9,13,15,33,35,36)]
    tXi[[i]] = test_gmert[test_gmert$DegreeName.out==levels(test_dataset$DegreeName.out)[i],c(6,8,9,13,15,33,35,36)]
    Zi[[i]]= as.matrix(rep(1,ni[i]))
    tZi[[i]]= as.matrix(rep(1,tni[i]))
    Yi[[i]] = as.matrix(dati_gmert$Status[dati_gmert$DegreeName.out==levels(train_dataset$DegreeName.out)[i]])
    tYi[[i]] = as.matrix(test_gmert$Status[test_gmert$DegreeName.out==levels(train_dataset$DegreeName.out)[i]])
  }
  
  q=1
  bizero <- list()
  length(bizero)<- 50		
  for(i in 1:50) bizero[[i]] <- matrix(0,nrow=q,ncol=1)
  
  gmert_mod = gmert(ni,  Xi=Xi, Zi=Zi,  Yi=Yi, tni=tni, tXi=tXi, tZi=tZi, tYi=tYi, xnam=xnam ,usesurrogate=2
                    ,surrogatestyle=0,maxsurrogate=0,maxcompete=2,minsplit=30,minbucket=10,maxdepth=4,xval=10
                    ,cp=0.006
                    ,F=1			#use F=50 to reproduce results in HAJJEM, et al.  # Chiara: micro-iter mi sembrano F*4
                    ,s.change=1e-6
                    ,mert.niter= 10		#use mert.niter=200 to reproduce results in HAJJEM, et al.
                    ,L=1			#use L=50  to reproduce results in HAJJEM, et al.
                    ,mert.verbose = TRUE ,sigmasqzero=1,Dzero=diag(x = 0.01 ,nrow=q,ncol=q),bizero=bizero	,family=binomial 		
                    ,rc= 1e-4,gmert.niter = 1		
                    ,S=1			#use S=55 to reproduce results in HAJJEM, et al.						
                    ,verbose = TRUE)
  
  
  test_pred_j <- gmert_mod[[1]]$gmert.t.c.prob
  test_pred_class_j <- ifelse(test_pred_j > p0opt,1,0)
  predicted_j = factor(test_pred_class_j, levels = c(0,1),labels = c('Dropout','Graduate'))
  observed_j = factor(test_dataset_j$Status)
  misc.table_j = table(predicted_j, observed_j)
  TP_j = misc.table_j[1,1]
  TN_j = misc.table_j[2,2]
  FP_j = misc.table_j[2,1]
  FN_j = misc.table_j[1,2]
  tot_j = dim(test_dataset_j)[1]
  accuracy_j[j] <- 1 - (FP_j+FN_j)/tot_j
  sensitivity_j[j] <- TP_j/(TP_j+FN_j)
  specificity_j[j] <- TN_j/(TN_j+FP_j)
}
avg_accuracy = mean(accuracy_j)
avg_accuracy 
avg_sensitivity = mean(sensitivity_j)
avg_sensitivity 
avg_specificity = mean(specificity_j)
avg_specificity 
sd(accuracy_j)
sd(sensitivity_j)
sd(specificity_j)




### Comparison with BiMM

train_dataset$Status = as.factor(ifelse(train_dataset$Status=='Graduate',1,0))
train_dataset$Sex = as.factor(train_dataset$Sex)
train_dataset$Nationality = as.factor(train_dataset$Nationality)
train_dataset$PrevStudies = as.factor(train_dataset$PrevStudies)
train_dataset$DegreeName.out = as.factor(train_dataset$DegreeName.out)

BiMM <- bimmTree(formula = Status ~ Sex +Nationality +PrevStudies +
                      AdmissionScore +AccessAge + WeiAvgEval1.1 +AvgAtt11 +
                      TotalCredits1.1, traindata = train_dataset,  random = train_dataset$DegreeName.out)
rpart.plot(BiMM$Tree)
rand_eff = BiMM$EffectModel

PVC = 0.3999^2 /(0.3999^2 + (pi^2)/3)
PVC

BiMM$EffectModel@u

rand_int_bimm = GMET$RandomEffects
rand_int_bimm$DegreeName.out$`(Intercept)` = BiMM$EffectModel@u

dotplot(rand_int_bimm,strip=T, lty= 4)

cor.test(GMET$RandomEffects$DegreeName.out$`(Intercept)`, rand_int_bimm$DegreeName.out$`(Intercept)`)


# ROC analysis

test_pred <- predict(BiMM$Tree,test_dataset,type="prob")[,2]
p0 = seq(0,1,0.001)
sensitivity <- specificity_comp <- NULL
i=0
for(k in p0){
  i = i+1
  pred2 <- ifelse(test_pred > p0[i],"Graduate","Dropout")
  t11 = length(which(pred2 =="Graduate"& test_dataset$Status=="Graduate"))
  t10 = length(which(pred2 =="Graduate" & test_dataset$Status=="Dropout"))
  t01 = length(which(pred2 =="Dropout" & test_dataset$Status=="Graduate"))
  t00 = length(which(pred2 =="Dropout" & test_dataset$Status=="Dropout"))
  specificity_comp[i] <- 1 - t11/(t11+t01)
  sensitivity[i] <- t00/(t00+t10) 
}
plot(specificity_comp, sensitivity, type = "l", xlab='1 - specificity', cex.lab = 1.3)
lines(seq(0,1,0.01), seq(0,1,0.01), lty = "dashed")
i0 = 301 # best ROC value
points(specificity_comp[i0], sensitivity[i0], pch = 1)
p0opt = p0[i0] # threshold value
p0opt

library(PRROC)
PRROC_obj <- roc.curve(scores.class0 = test_pred, weights.class0=ifelse(test_dataset$Status=='Graduate',1,0),
                       curve=TRUE)
plot(PRROC_obj)


# ONE SHOT indexes
test_pred_class <- ifelse(test_pred > p0opt,1,0)
predicted = factor(test_pred_class, levels = c(0,1),labels = c('Dropout','Graduate'))
observed = factor(test_dataset$Status)
misc.table = table(predicted, observed)
misc.table

TP = misc.table[1,1]
TN = misc.table[2,2]
FP = misc.table[2,1]
FN = misc.table[1,2]
tot = dim(test_dataset)[1]

misClasificError <- (FP+FN)/tot
print(paste('Accuracy',round(1-misClasificError,4)))
print(paste('Sensitivity',round(TP/(TP+FN),4)))
print(paste('Specificity',round(TN/(TN+FP),4)))








